/*
   THE KID CLOCK
   by PAPY ET LES RESISTANCES adapted from Andy Doro (Many thanks to him)

   A clock using RGB LEDs for a color shift effect. 
   Added push buttons to set hour and minute, and potentiometer to adjust brightness

   Hardware:
   - Arduino Nano
   - DS3231 RTC breakout
   - LEDs....


   Software:

   This code requires the following libraries:

   - RTClib https://github.com/adafruit/RTClib
   - DST_RTC https://github.com/andydoro/DST_RTC


   Wiring:
   - Solder DS3231 breakout to Arduino Nano, A2 to GND, A3 to PWR, A4 to SDA, A5 to SCL
   - hour push button to D4 and GND
   - minute push button to D5 and GND
   - brightness potentiometer to GND, A0, 5V 
   - LEDs in to D9
  
*/

// include the library code:
#include <Wire.h>
#include <RTClib.h>
#include <DST_RTC.h>
#include <FastLED.h>
#include <Ephemeris.h>


// Software version
#define VERSION "1.0.2"

#define DEBUG false

// Location of clock: used to compute sunset and sunrise time
// Coordinates for Paris
#define LAT 48.856614
#define LONG 2.3522219
#define UTC_OFFSET 1

// LEDs are connected together: first the 24 LEDs large ring for hours, then the 12 LEDs small ring for minutes, then the individual items
// Mapping of LEDs: 24 LEDS for 12 hours
#define LED_HOUR_BEGIN  0   	// LED index for hour 1
#define LED_HOUR_END  	23 		// 24 LEDs, every other ones used for hours
// Mapping of LEDs: 12 LEDS for 60 minutes (1 LED represents 5 minutes)
#define LED_MIN_BEGIN  	24  // LED index for minute 5
#define LED_MIN_END		 35		// Last LED for minute 55
#define LED_STAR_1  36      // LED index for the 1st star
// Moon
#define LED_MOON     37      // LED index for the moon// Stars
#define LED_STAR_2  38      // LED index for the 1st star
#define LED_STAR_3  39      // LED index for the 1st star
#define LED_STAR_4  40      // LED index for the 1st star
#define LED_STAR_5  41      // LED index for the 1st star
#define NB_STARS 5
// Sun
#define LED_SUN 		42
#define LED_SUNRISE		43
#define LED_OWL     44
#define LED_CAT     45
#define LED_SUNSET		46
// Total number of LEDs
#define NUM_LEDS 		47

byte stars[NB_STARS] = { LED_STAR_1,  LED_STAR_2, LED_STAR_3, LED_STAR_4, LED_STAR_5 };

#define FRAMES_PER_SECOND  120

// define pins
#define LEDPIN 9  // use to control the LEDs
// Clock DS3231 connections
// GND: A2
// VCC: A3
// SDA: A4
// SCL: A5
#define RTCGND A2 // use this as DS3231 breakout ground 
#define RTCPWR A3 // use this as DS3231 breakout power

// Push buttons
#define H_BTN_PIN 4  // Hour button
#define M_BTN_PIN 5  // Minute button

// LDR input
#define LDR_PIN A1
// Potentiometer for brightness settings
#define POT_PIN A0   // mid pin potentiometer

// max brightness based on time of day- could try warmer colors at night?
#define MINBRIGHTNESS 0
#define MAXBRIGHTNESS 255
#define VARBRIGHTNESS 100

#define DAYBRIGHTNESS 100  // 40
#define MINBRIGHTNESS 40

#define SUNSET_DURATION  15   // in minutes
#define SUNRISE_DURATION 15

// Definition of some color palettes
DEFINE_GRADIENT_PALETTE( sun_gradient_palette ) {
  0,     0,  0,  0,   //black
128,   255,  0,  0,   //red
200,   255,255,  0 };  //bright yellow

CRGBPalette16 sunPalette = sun_gradient_palette;

uint8_t gHue = 0; // rotating "base color" used by many of the patterns


/*****************************************************************/

// Establish clock object
RTC_DS3231 RTC;

DateTime theTime;  // Holds current clock time
TimeSpan fiveMinutes;


// Define the array of leds
CRGB leds[NUM_LEDS];

// Brightness variable
int Brightness = MINBRIGHTNESS;

// Daylight saving time not taken into account
#define OBSERVE_DST 0

enum phaseOfDay { NIGHT, DAWN, DAY, DUSK };

typedef struct MOMENTOFDAY {
  phaseOfDay phase;
  int percent;
} momentOfDay;


/*******************************************************************************************/
void setup() {
  // put your setup code here, to run once:

  //Serial for debugging
  Serial.begin(9600);

  // Print version and date of compilation
  Serial.println();
  Serial.print("Version: ");
  Serial.println(VERSION);
  Serial.print("Compiled on ");
  Serial.print(__DATE__);
  Serial.print(" - ");
  Serial.println(__TIME__); 
  Serial.println("Enter YYMMDDHHMM to set time");

  // set pinmodes
  pinMode(LEDPIN, OUTPUT);

  // set analog pins to power DS1307 breakout!
  pinMode(RTCGND, OUTPUT); // analog 2
  pinMode(RTCPWR, OUTPUT); // analog 3

  // set them going!
  digitalWrite(RTCGND, LOW);  // GND for RTC
  digitalWrite(RTCPWR, HIGH); // PWR for RTC

  // push buttons
  pinMode(H_BTN_PIN, INPUT);
  digitalWrite(H_BTN_PIN, HIGH);
  pinMode(M_BTN_PIN, INPUT);
  digitalWrite(M_BTN_PIN, HIGH);

  // potentiometer for brightness
  pinMode(POT_PIN, INPUT);

  // LDR for light measurement
  pinMode(LDR_PIN, INPUT);
  
   // init display
  FastLED.addLeds<WS2812, LEDPIN, RGB>(leds, NUM_LEDS);

  FastLED.clear();
  FastLED.show();

  // test LEDs
  testLEDs();

 // start clock
  Wire.begin();  // Begin I2C
  RTC.begin();   // begin clock
 

}

/*******************************************************************************************/
void loop() {

 // If something is coming in on the serial line, it's
  // a time correction so set the clock accordingly.
  if (Serial.available()) {    
    ReadSerialAndSetTime();
  }

  // If Hour push button is down, add 1 hour
  if (btnPushed(H_BTN_PIN)) {
    incHour();
  }

  // If Minute push button is down, add 1 minute
  if (btnPushed(M_BTN_PIN)) {
    incMinute();
  }
   
  // get the time
  theTime = RTC.now();

  // Adjust brightness according to the ambient light 
  adjustBrightness();
  
  displayTime();

  FastLED.delay(1000 / FRAMES_PER_SECOND);
}

/*******************************************************************************************/
/* .                                      Test LEDS                                        */
/*******************************************************************************************/  
#define ARRAY_SIZE(A) (sizeof(A) / sizeof((A)[0]))
// List of patterns to cycle through.  Each is defined as a separate function below.
typedef void (*SimplePatternList[])();
SimplePatternList gPatterns = { confetti, sinelon, juggle };
uint8_t gCurrentPatternNumber = 0; // Index number of which pattern is current

void testLEDs() {
  
  unsigned long StartTime = millis();       // in ms
  unsigned long ElapsedTime;
 
  do {
    // Call the current pattern function once, updating the 'leds' array
    gPatterns[gCurrentPatternNumber]();

    // send the 'leds' array out to the actual LED strip
    FastLED.show();  
    // insert a delay to keep the framerate modest
    FastLED.delay(1000/FRAMES_PER_SECOND); 

    // do some periodic updates
    EVERY_N_MILLISECONDS( 20 ) { gHue++; } // slowly cycle the "base color" through the rainbow
    EVERY_N_SECONDS( 5 ) { nextPattern(); } // change patterns periodically
    ElapsedTime = millis() - StartTime;
  } while (ElapsedTime < 10000);
  
}

void nextPattern()
{
  // add one to the current pattern number, and wrap around at the end
  gCurrentPatternNumber = (gCurrentPatternNumber + 1) % ARRAY_SIZE( gPatterns);
}

void rainbow() 
{
  // FastLED's built-in rainbow generator
  fill_rainbow( leds, NUM_LEDS, gHue, 7);
}

void rainbowWithGlitter() 
{
  // built-in FastLED rainbow, plus some random sparkly glitter
  rainbow();
  addGlitter(80);
}

void addGlitter( fract8 chanceOfGlitter) 
{
  if( random8() < chanceOfGlitter) {
    leds[ random16(NUM_LEDS) ] += CRGB::White;
  }
}

void confetti() 
{
  // random colored speckles that blink in and fade smoothly
  fadeToBlackBy( leds, NUM_LEDS, 10);
  int pos = random16(NUM_LEDS);
  leds[pos] += CHSV( gHue + random8(64), 200, 255);
}

void sinelon()
{
  // a colored dot sweeping back and forth, with fading trails
  fadeToBlackBy( leds, NUM_LEDS, 20);
  int pos = beatsin16( 13, 0, NUM_LEDS-1 );
  leds[pos] += CHSV( gHue, 255, 192);
}

void bpm()
{
  // colored stripes pulsing at a defined Beats-Per-Minute (BPM)
  uint8_t BeatsPerMinute = 62;
  CRGBPalette16 palette = PartyColors_p;
  uint8_t beat = beatsin8( BeatsPerMinute, 64, 255);
  for( int i = 0; i < NUM_LEDS; i++) { //9948
    leds[i] = ColorFromPalette(palette, gHue+(i*2), beat-gHue+(i*10));
  }
}

void juggle() {
  // eight colored dots, weaving in and out of sync with each other
  fadeToBlackBy( leds, NUM_LEDS, 20);
  byte dothue = 0;
  for( int i = 0; i < 8; i++) {
    leds[beatsin16( i+7, 0, NUM_LEDS-1 )] |= CHSV(dothue, 200, 255);
    dothue += 32;
  }
}
